package com.javaranch.common ;

import java.io.* ;

/** Date is tracked as an integer. <p>

    Technically, a Julian calendar is the calendar designed by
    Julius Caesar in 46 B.C.  It has months and leap years much like
    the gregorian calendar we use now (designed by Pope Gregory the
    13th in 1582).  But for several years, computer geeks have counted
    the number of days since the beginning of a year or another point
    in time and referred to this as a "Julian Date".  From here on
    out, any reference to a "julian date" means "the number of days
    since January 1, 0000".  This class will behave as if the
    Gregorian calendar leap year system has been in place since
    January 1, 0000.<p>

    This class will behave as both a Julian date and a Gregorian date
    although the actual date will be stored as a single integer.<p>

    
    - - - - - - - - - - - - - - - - - <p>
    
    Copyright (c) 1998-2000 Paul Wheaton <p>
    
    You are welcome to do whatever you want to with this source file provided
    that you maintain this comment fragment (between the dashed lines).
    Modify it, change the package name, change the class name ... personal or business 
    use ...  sell it, share it ... add a copyright for the portions you add ... <p>
    
    My goal in giving this away and maintaining the copyright is to hopefully direct 
    developers back to JavaRanch. <p>
    
    The original source can be found at <a href=http://www.javaranch.com>JavaRanch</a> <p>
    
    - - - - - - - - - - - - - - - - - <p>

    @author Paul Wheaton
*/

public class JDate implements Serializable
{
    private int j ; //  number of days since Jan 1, 0000

    /** Create a new JDate object with a default of zero. <p>
    */
    public JDate()
    {
        j = 0 ;
    }

    /** Create a new JDate object specified as the number of days since January 1, 0000. <p>
    */
    public JDate( int julianDate )
    {
        j = julianDate ;
    }

    /** Create a new JDate object equal to another JDate object. <p>
    */
    public JDate( JDate julianDate )
    {
        j = julianDate.j ;
    }

    /** Create a new JDate object based on a year, month and day. <p>
    */
    public JDate( int year , int month , int day )
    {
        j = convertGregorianToJulian( year , month , day );
    }

    /** Create a new JDate object equal to a GDate object. <p>
    */
    public JDate( GDate g )
    {
        j = convertGregorianToJulian( g.getYear() , g.getMonth() , g.getDay() );
    }

    private static boolean isLeapYear( int year )
    {
        boolean returnVal = false ;
        if ( year % 400 == 0 )
        {
            // this is definitely a leap year
            returnVal = true ;
        }
        else if ( year % 4 == 0 )
        {
            // this is probably a leap year
            if ( year % 100 == 0 )
            {
                // this is definitely not a leap year
                // do nothing
            }
            else
            {
                // this is definitely a leap year
                returnVal = true ;
            }
        }
        return returnVal ;
    }

    private static final int daysInFourYears     = 1461 ;   // number of days in four years
    private static final int daysInCentury       = 36524 ;  // number of days in a century
    private static final int daysInFourCenturies = 146097 ; // number of days in four centuries

    private static final int[] daysInMonth = {31,28,31,30,31,30,31,31,30,31,30,31} ;

    /** Report if the given date is valid or not. <p>
    */
    public static boolean isValid( GDate date )
    {
        boolean returnVal = false ;
        int year = date.getYear();
        if ( Numbers.inRange( year , 1200 , 2200 ) )
        {
            int month = date.getMonth() ;
            if ( Numbers.inRange( month , 1 , 12 ) )
            {
                int day = date.getDay() ;
                if ( day >= 1 )
                {
                    if ( day <= daysInMonth[ month - 1 ] )
                    {
                        returnVal = true ;
                    }
                    else if ( ( month == 2 ) && ( day == 29 ) && isLeapYear( year ) )
                    {
                        returnVal = true ;
                    }
                }
            }
        }
        return returnVal ;
    }

    private static GDate convertJulianToGregorian( int x )
    {
        // x is a disposable variable I'll chip at to find the gregorian values
        int numFC = x / daysInFourCenturies ;  // number of "four century" chunks
        int y = numFC * 400 ;
        x -= ( daysInFourCenturies * numFC ) ;
        int numCenturies = 0 ;
        if ( x > ( daysInCentury + 1 ) )
        {
            x -= ( daysInCentury + 1 ) ;
            y += 100 ;
            numCenturies = x / daysInCentury ;
            y += ( numCenturies * 100 );
            x -= ( numCenturies * daysInCentury );
            numCenturies++;
        }
        if ( ( numCenturies > 0 ) && ( x > ( daysInFourYears - 1 ) ) )
        {
            x -= ( daysInFourYears - 1 );
            y += 4 ;
        }
        long numFY = x / daysInFourYears ; // number of "four year" chunks
        y += ( numFY * 4 );
        x -= ( numFY * daysInFourYears );
        int m = 0 ;
        int d = 0 ;
        if ( isLeapYear( y ) )
        {
            if ( x >= 366 )
            {
                x -= 366;
                y++;
            }
            else if ( x == 59 ) // Jan 1, would have x = 0, Feb 29 would have x=59
            {
                m = 2 ;
                d = 29 ;
            }
            else if ( x > 59 )
            {
                x--;
            }
        }
        if ( m == 0 ) // otherwise, g was set as a leap day
        {
            while ( x >= 365 )
            {
                x -= 365 ;
                y++;
            }
            while( daysInMonth[ m ] <= x )
            {
                x -= daysInMonth[ m ];
                m++;
            }
            m++;
            d = x + 1 ;
        }
        return new GDate( y , m , d ) ;
    }

    private static int convertGregorianToJulian( int y , int m , int d )
    {
        boolean leap = isLeapYear( y - ( y % 4 ) );
        int numFC = y / 400 ; // the number of "four century" chunks
        y -= numFC * 400 ;
        int j = numFC * daysInFourCenturies ; // the value to be returned
        int numCenturies = y / 100 ;
        if ( numCenturies > 0 )
        {
            y -= numCenturies * 100 ;
            j += ( numCenturies * daysInCentury ) + 1 ;  // the first century has one more day
        }
        int numFY = y / 4 ; // the number of "four year" chunks
        y -= numFY * 4 ;
        if ( ( numCenturies != 0 ) && ( numFY > 0 ) )
        {
            // generally, the first four year chunk doesn't have a leap year
            j += 1460 ;
            numFY--;
        }
        j += numFY * daysInFourYears ;
        if ( leap )
        {
            if ( y > 0 )
            {
                j += 366 ;
                y--;
            }
            else if ( m > 2 )
            {
                j++;
            }
        }
        while ( y > 0 )
        {
            j += 365 ;
            y--;
        }
        int i ;
        for ( i = 0 ; i < m - 1 ; i++ )
        {
            j += daysInMonth[ i ];
        }
        j += ( d - 1 ) ;
        return j ;
    }

    private static int convertGregorianToJulian( GDate g )
    {
        return convertGregorianToJulian( g.getYear() , g.getMonth() , g.getDay() );
    }

    /** Set this object to be equal to an integer. <p>
    */
    public void set( int newJulianDate )
    {
        j = newJulianDate ;
    }

    /** Set this object to be equal to another JDate object. <p>
    */
    public void set( JDate newJulianDate )
    {
        j = newJulianDate.j ;
    }

    /** Set this object to be equal to a GDate object. <p>
    */
    public void set( GDate g )
    {
        j = convertGregorianToJulian( g ) ;
    }

    /** Change just the year - the month and day values will be unmodified. <p>
    */
    public void setYear( int newYear )
    {
        GDate g = convertJulianToGregorian( j );
        g.setYear( newYear );
        j = convertGregorianToJulian( g );
    }

    /** Change just the month - the year and day values will be unmodified. <p>
    */
    public void setMonth( int newMonth )
    {
        GDate g = convertJulianToGregorian( j );
        g.setMonth( newMonth );
        j = convertGregorianToJulian( g );
    }

    /** Change just the day - the year and month values will be unmodified. <p>
    */
    public void setDay( int newDay )
    {
        GDate g = convertJulianToGregorian( j );
        g.setDay( newDay );
        j = convertGregorianToJulian( g );
    }

    /** Get the integer representation. <p>
    */
    public int get()
    {
        return j ;
    }

    /** Extract the year. <p>
    */
    public int getYear()
    {
        GDate g = convertJulianToGregorian( j );
        return g.getYear() ;
    }

    /** Extract the month. <p>
    */
    public int getMonth()
    {
        GDate g = convertJulianToGregorian( j );
        return g.getMonth() ;
    }

    /** Get the full month name:  "January", "February", "March", etc. <p>
    */
    public String getMonthString()
    {
        return GDate.getMonthString( getMonth() );
    }

    /** Get the day of the month (not the day of the week). <p>
    */
    public int getDay()
    {
        GDate g = convertJulianToGregorian( j );
        return g.getDay() ;
    }

    /** Get the day of the week (not the day of the month). <p>

        Zero is monday. <p>
    */
    public int getDOW()
    {
        return ( ( j - 2 ) % 7 ) ;
    }

    private static final String[] dowName =
        {"Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday"};

    /** Get the full day of the week name:  "Monday", "Tuesday", "Wednesday", etc. <p>
    */
    public String getDOWString()
    {
        return dowName[ getDOW() ];
    }

    /** Convert to a GDate object. <p>
    */
    public GDate getGDate()
    {
        return convertJulianToGregorian( j );
    }

    /** Get something like "05-01-1999" for May 5th, 1999. <p>
    */
    public String getShortFormat()
    {
        GDate g = convertJulianToGregorian( j );
        return g.getShortFormat() ;
    }

    /** Get something like "Wednesday, September 22, 1999". <p>
    */
    public String getLongFormat()
    {
        StringBuffer buffy = new StringBuffer( getDOWString() );
        buffy.append(", ");
        GDate g = convertJulianToGregorian( j );
        buffy.append( g.getLongFormat() );
        return buffy.toString();
    }

    /** Compare this object to an integer. <p>
    */
    public boolean equals( int julianDate )
    {
        return ( julianDate == j );
    }

    /** Compare this object to another JDate object. <p>
    */
    public boolean equals( JDate julianDate )
    {
        return ( julianDate.j == j );
    }

    /** Increment the date by one. <p>
    */
    public void inc()
    {
        j++ ;
    }

    /** Increment the date by the given value. <p>

        Negative values will decrement. <p>
    */
    public void inc( int adjust )
    {
        j += adjust ;
    }

    /** Decrement the date by one. <p>
    */
    public void dec()
    {
        j-- ;
    }

    /** Adjust the date by the given value. <p>

        Positive values will increment. <p>

        Negative values will decrement. <p>
    */
    public void adjust( int adj )
    {
        j += adj ;
    }

    /** Provided for testing purposes only. <p>
    */
    public static void unitTest( ErrorLog log )
    {
        // Convert a julian date to a gregorian date and then back to a julian date.
        // The two julian dates should be the same.  Do this test for 10,000 consecutive dates.
        GDate today = new GDate();
        JDate j = new JDate( today );
        j.adjust( -10000 );
        for( int i = 0 ; i < 10000 ; i++ )
        {
            int orig = j.get();
            GDate g = j.getGDate();
            j.set( g );
            if ( orig != j.get() )
            {
                log.add( "date conversion failure at " + orig );
            }
            j.set( orig + 1 );
        }
    }

}



